/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.kernel.axiom.atomic;

import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.atomic.shaper.Filter;
import com.vii.brillien.kernel.axiom.atomic.shaper.Shifter;
import com.vii.brillien.kernel.axiom.transport.Communication;

/**
 * Context is a special Unit type providing a metalevel collapsing set functionality extended by sharing services.
 */
public interface Context <P extends Presence, C extends Communication, R> extends Unit<P,C,R>{

    /**
     * Shares an object with the given key
     * @param p component reference who want to share
     * @param key unique key for the sharing
     * @param object shared object
     * @param <T> shared content's type
     */
    <T> void                registerSharing( Presence p, Object key, T object );

    /**
     * Removes a shared content
     * @param p component reference who want to share
     * @param key unique key for the sharing
     */
    void                    removeSharing( Presence p, Object key );

    /**
     * Decides whether a sharing is available in this flow. This may induce a searching in the parent flows too.
     * @param p searcher component reference
     * @param key key of the sharing that should be retrieved
     * @return availability of a sharing by a given name
     */
    boolean                 hasSharing( Presence p, Object key );

    /**
     * Getter method to access a sharing by a given name
     * @param p accessor Presence reference
     * @param key key of the shared content
     * @param <S> shared content
     * @return shared content's type
     */
    <S> S                   getSharing( Presence p, Object key );


    /**
     * Transforms the given exception according to the filter-shifter mappings available in this context
     */
    BrillienException       levelException( BrillienException be );

    /**
     * Gets the proper Shifter to the given BrillienException instance if exists
     */
    Shifter                 getShifter( BrillienException be );

    /**
     * Adds a Filter-Shifter mapping to the given context
     */
    void                    addExceptionLeveling( Filter f, Shifter s );

    /**
     * Removes a Filter-Shifter mapping by the given filter
     */
    void                    removeExceptionLeveling( Filter f );

    /**
     * Clears all Filter-Shifter mappings from the given context
     */
    void                    clearExceptionLeveling( );

}
